import {Grid} from "$lib/grid";
import type {ObjectValues} from "$lib/components/object-values";
import {THRUSTER_LIST, THRUSTER_TYPE_LIST} from "$lib/thruster.svelte";
import {applyUnits} from "$lib/constants";

type InventoryDetails = {
    key: string,
    name: string,
    capacity: Map<Grid, number>
}

export const INVENTORIES: { [key: string]: InventoryDetails } = {
    CargoLarge: {
        key: "CargoLarge",
        name: "largeCargo",
        capacity: new Map([
            [Grid.Small, 15625],
            [Grid.Large, 421000],
        ])
    },
    CargoMedium: {
        key: "CargoMedium",
        name: "mediumCargo",
        capacity: new Map([
            [Grid.Small, 3375],
        ])
    },
    CargoSmall: {
        key: "CargoSmall",
        name: "smallCargo",
        capacity: new Map([
            [Grid.Small, 125],
            [Grid.Large, 15625],
        ])
    },
    Connector: {
        key: "Connector",
        name: "connector",
        capacity: new Map([
            [Grid.Small, 1152],
            [Grid.Large, 8000],
        ])
    }
}

type InventoryList = ObjectValues<typeof INVENTORIES>;

export class Inventory {
    details: InventoryDetails;

    constructor(details: InventoryDetails) {
        if (typeof details === "undefined") {
            throw("Inventory undefined, cannot create Object");
        }
        this.details = details;
    }

    getVolume(grid: Grid, multiplier: number): number {
        return (this.details.capacity.get(grid) || 0) * multiplier;
    }
}